/**************************************************************

comfun.c (C-Munipack project)
Common functions, macros, etc.
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifndef _WIN32
#include <sys/stat.h>
#include <sys/types.h>
#endif

#include "comfun.h"

int cmpack_quiet = 0;					
	// Quiet output flag
int cmpack_debug = 0;
	// Verbose output flag
cmpack_cbtype *cmpack_cbproc = NULL;
	// Output callback procedure

// Prints short version of GPL license to the stdout.
void PrintLicense()
{
 printf("This program is free software; you can redistribute it and/or modify\n");
 printf("it under the terms of the GNU General Public License, version 2\n");
 printf("as published by the Free Software Foundation.\n\n");
 
 printf("This program is distributed in the hope that it will be useful\n");
 printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
 printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
 printf("GNU General Public License for more details.\n");
 printf("You should have received a copy of the GNU General Public License\n");
 printf("along with this program; if not, write to the Free Software\n");
 printf("Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n");
}

void PrintVersion(const char *program)
{
 printf("%s (%s) %s\n\n", program, PKGNAME, VERSION);
 printf(COPYRIGHTLINE);
 printf(WARRANTYLINE);
 printf(HOMEPAGELINE);
 printf(BUGREPORTS);
}

void PrintPrologue(const char *program)
{
 if (!cmpack_quiet) 
 	printf("%s (%s) %s\n\n", program, PKGNAME, VERSION);
}

// This function translates error code to its human readable representation. The string
// is stored to 'buf' string, buflen specifies the length of buffer in bytes. Status is
// error code.
void cmpack_formaterror(char *buf, int buflen, int status)
{
  switch (status) {
	case MPK_MEMORY:
	    strncpy(buf,"Insufficient memory", buflen);
	    break;
	case MPK_CANNOT_OPEN_SRC:
	    strncpy(buf,"Cannot open the source file", buflen);
	    break;
	case MPK_CANNOT_OPEN_OUT:
	    strncpy(buf,"Cannot open the destination file", buflen);
	    break;
	case MPK_CANNOT_OPEN_PAR:
	    strncpy(buf,"Cannot open the parameter file", buflen);
	    break;
    case MPK_NO_DIRFILE:
	    strncpy(buf,"Dirfile not found", buflen);
		break;
    case MPK_NO_INPUT_FILES:
	    strncpy(buf,"No input files", buflen);
		break;
    case MPK_INVALID_SIZE:
	    strncpy(buf,"Invalid dimensions of image", buflen);
		break;
    case MPK_INVALID_PAR:
	    strncpy(buf,"Invalid value of parameter", buflen);
		break;
    case MPK_INVALID_DATA:
	    strncpy(buf,"Invalid data in source file", buflen);
		break;
    case MPK_NO_DATA:
	    strncpy(buf,"No data loaded in memory", buflen);
		break;
    case MPK_UNKNOWN_FORMAT:
	    strncpy(buf,"Unknown format of source file", buflen);
		break;
    case MPK_INVALID_DATE:
	    strncpy(buf,"Invalid format of date or time of observation", buflen);
		break;
	case MPK_INVALID_HEADER:
	    strncpy(buf,"Error in header of the source file", buflen);
		break;
	case MPK_KEY_NOT_FOUND:
	    strncpy(buf,"Key not found", buflen);
		break;
    case MPK_DATA_UNDERFLOW:
        strncpy(buf,"An underflow has been occurred during computation", buflen);
        break;
    case MPK_DATA_OVERFLOW:
        strncpy(buf,"An overflow has been occurred during computation", buflen);
        break;
	case MPK_CLOSED_FILE:
		strncpy(buf,"Operation not allowed on closed file", buflen);
		break;
	case MPK_ERROR:
		strncpy(buf,"General error", buflen);
		break;
	case MPK_FRAME_NOT_FOUND:
	    strncpy(buf,"Frame not found", buflen);
		break;
	case MPK_INVALID_OBJRA:
	    strncpy(buf,"Invalid format of right ascension", buflen);
		break;
	case MPK_INVALID_OBJDEC:
	    strncpy(buf,"Invalid format of declination", buflen);
		break;
	case MPK_INVALID_OBSLON:
	    strncpy(buf,"Invalid format of longitude", buflen);
		break;
	case MPK_INVALID_OBSLAT:
	    strncpy(buf,"Invalid format of latitude", buflen);
		break;
	case MPK_MISSING_OBJCOORDS:
	    strncpy(buf,"Missing object's equatorial coordinates", buflen);
		break;
	case MPK_MISSING_OBSCOORDS:
	    strncpy(buf,"Missing observer's geographical coordinates", buflen);
		break;
	case MPK_INVALID_EXPTIME:
	    strncpy(buf,"Invalid value of exposure duration", buflen);
		break;

    case DRK_CANNOT_OPEN_DRK:
	    strncpy(buf,"Dark frame not found", buflen);
		break;
    case DRK_DIFFERENET_SIZE:
	    strncpy(buf,"Dimensions of dark frame and scientific image are different", buflen);
		break;
	case DRK_INVALID_EXPTIME:
	    strncpy(buf,"Invalid value of exposure duration in dark frame file", buflen);
		break;
	
    case FLT_CANNOT_OPEN_FLT:
	    strncpy(buf,"Flat frame not found", buflen);
		break;
    case FLT_DIFFERENET_SIZE:
	    strncpy(buf,"Dimensions of flat frame and scientific image are different", buflen);
		break;
	case FLT_MEAN_ZERO:
	    strncpy(buf,"Mean value of flat frame is zero", buflen);
		break;
    case FLT_DIVZERO:
	    strncpy(buf,"Zero value has been occurred on flat frame (can't divide by zero)", buflen);
		break;

    case MAT_CANNOT_OPEN_REF:
	    strncpy(buf,"Reference file was not found", buflen);
		break;
    case MAT_FEW_POINTS_REF:
	    strncpy(buf,"Too few stars in the reference file", buflen);
		break;
    case MAT_FEW_POINTS_SRC:
	    strncpy(buf,"Too few stars in the source file", buflen);
		break;
    case MAT_MATCH_NOT_FOUND:
	    strncpy(buf,"Coincidences not found", buflen);
		break;
    case MAT_XML_PARSE:
	    strncpy(buf,"Syntax error in the catalogue file", buflen);
		break;

    case AFL_DIFFERENET_SIZE:
    case MDK_DIFFERENET_SIZE:
	    strncpy(buf,"Input frames are not compatible (different sizes)", buflen);
		break;
	case MDK_CANNOT_OPEN_BIAS:
	case MPK_CANNOT_OPEN_BIAS:
		strncpy(buf,"Bias frame not found", buflen);
		break;
	case MDK_DIFF_BIASSIZE:
		strncpy(buf,"Dimensions of bias frame and dark frame are different", buflen);
		break;
	case MPK_DIFF_BIAS_SIZE:
	    strncpy(buf,"Dimensions of bias frame and scientific image are different", buflen);
		break;

	case MFI_TOO_FEW_DATA:
	    strncpy(buf,"Too few data in source files", buflen);
		break;
	case MFI_REF_NOT_FOUND:
	    strncpy(buf,"Reference star was not found", buflen);
		break;

    default:
       strncpy(buf, "Unknown error status", buflen);
       break;
	}
}

// This internal functions should be used for all print outs. It performs message
// filtering and pass it to the callback procedure.
void PrintOut(int debug, const char *message)
{
	if (cmpack_cbproc && !cmpack_quiet && (!debug || cmpack_debug))
		cmpack_cbproc(message);
}

// Default callback function which prints messages to stdout.
void PrintMessage(const char *msg)
{
	if (!cmpack_quiet)
 		printf("%s\n",msg);
}

// Prints human readable form of error code to the specified file
void PrintError(const char *program, const char *file, int aline, int status)
{
	char buf[MAXLINE];

	cmpack_formaterror(buf,MAXLINE,status);
	if (program) {
		if (file) {
			if (aline>0) 
				fprintf(stderr,"%s:%s:%d:%s\n", program, file, aline, buf);
			else 
				fprintf(stderr,"%s:%s:%s\n", program, file, buf);
		} else          
			fprintf(stderr,"%s:%s\n", program, buf);
	} else              
		fprintf(stderr,"%s\n", buf);
}

// Sets output level. Returns previous level
int cmpack_setoutput(cmpack_cbtype *cbproc, int level)
{
	int retval = (cmpack_quiet ? MPK_LEVEL_QUIET : 
	             	(cmpack_debug ? MPK_LEVEL_DEBUG : MPK_LEVEL_NORMAL));
	cmpack_quiet = (level == MPK_LEVEL_QUIET);
	cmpack_debug = (level == MPK_LEVEL_DEBUG);
	cmpack_cbproc = cbproc;
	return retval;
}

// Returns CMunipack's version identifier
void cmpack_getversionid(char *buf, int buflen)
{
	strncpy(buf,VERSION,buflen);
}

// Makes copy of specified file (infile to outfile)
// Returns zero on success, negative value on failure.
int copyfile(const char *infile, const char *outfile)
{
  char buf[MAXBUFFER];
  FILE *in, *out;
  size_t size;

  /* Open source file */
  in = fopen(infile, "rb");
  if (in==0)
      return -1;

  /* Open destination file */
  out = fopen(outfile, "wb");
  if (out==0) {
	  fclose(in);
	  return -2;
  }

  /* Copy data */
  while (!feof(in)) {
	  size = fread(buf, 1, MAXBUFFER, in);
	  fwrite(buf, 1, size, out);
  }

  /* Close files and return */
  fclose(in);
  fclose(out);
  return 0;
}

#ifndef _WIN32
int chkdir(const char *path)
{
  struct stat buf;

  if (stat(path, &buf)==0) {
  	// Directory or file exists
  	return (S_ISDIR(buf.st_mode) ? 1 : 0);
  } else {
  	if (mkdir(path, S_IREAD | S_IWRITE | S_IEXEC | S_IRGRP | S_IROTH)==0)
  		return 1;
  }
  return 0;
}
#endif

void MaskToFormat(char *format, const char *mask) {
/* Converts output mask to formatting string */
	char *j;
	int n;
	char prefix[MAXLINE], suffix[MAXLINE];

    /* remove '*' wildcard output mask - replace by single '?' */
    j = strchr(mask,'*');
    if (j!=NULL) *j='?';

    j = strchr(mask,'?');
    if (j!=NULL) {
        strncpy(prefix,mask,j-mask);               /* chars before ? marks */
        prefix[j-mask] = 0;
        for (n=0; *j=='?'; j++) n++;               /* n = number of ? marks */
        strncpy(suffix,j,MAXLINE);                  /* chars after ? marks */
        sprintf(format,"%s%%0%dd%s",prefix,n,suffix);
    } else {
	    strcpy(format,mask);
    }
}

char *strup(char *s)
/* Converts all chars in s to upper case */
{
  for( ; *s != '\0'; s++ )
    if( *s >= 'a' && *s <= 'z' )
      *s = *s - ('a' - 'A');
  return s;
}

char *changeext(char *outf, const char *inf, const char *ext)
/* Changes extension in file name */
{
	char *x;
	strcpy(outf,inf);
	if ((x = strrchr(outf,'.'))!=NULL) x[0] = 0;
    strcat(outf,".");
	strcat(outf,ext);
	return outf;
}

const char *memcasestr(const char *haystack, size_t haystacklen, const char *needle)
// Finds a string needle in array haystack (uses memmem)
{
	size_t i, j;
	const char *aux, *s1, *s2;

	for (i=0, aux=haystack; i<haystacklen; i++, aux++) {
		if (toupper(*aux) == toupper(*needle)) {
			s1 = aux; s2 = needle; j = i;
			do {
				s1++; s2++; j++;
				if (*s2=='\0') return aux;
				if (j>=haystacklen) return NULL;
			} while (toupper(*s1)==toupper(*s2));
		}
	}
	return NULL;
}

const char *cmpack_shareddir(void)
{
#ifndef _WIN32
	return DATADIR "/cmunipack";
#else
	return "";
#endif
}
