/**************************************************************

convert.c (C-Munipack project)
Conversion functions
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>

#include "comfun.h"
#include "convert.h"

// Converts string to double (with default value)
double getvalf(const char *buf, double defval)
{
	char *stpos, *endpos;
	double x = defval;

	stpos = strchr(buf,'=');
	if (stpos!=NULL) {
		stpos++;
  	    x = strtod(stpos,&endpos);
        if (endpos==stpos) x=defval;
    }
    return x;
}

// Converts string to longint (with default value)
long getvall(const char *buf, long defval)
{
	char *stpos, *endpos;
	long x = defval;

	stpos = strchr(buf,'=');
	if (stpos!=NULL) {
		stpos++;
  	    x = strtol(stpos,&endpos,10);
        if (endpos==stpos) x=defval;
    }
    return x;
}

// Converts the string to R.A. The value is stored to the 'ra' parameter.
// Returns zero on success or MPK_INVALID_PAR error code on failure.
int cmpack_strtora(const char *buf, double *ra)
{
  if (rd_parse(buf, ra)==0) {
    if (*ra>=0.0 && *ra<24.0)
      return 0;
    else
      return MPK_INVALID_PAR;
 } else
   return MPK_INVALID_PAR;
}

void cmpack_ratostr(double ra, char *buf)
/* Converts the R.A. into the string */
{
  long x = (long)(ra*3600.0+0.5);
  sprintf(buf, "%ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
}

int cmpack_strtodec(const char *buf, double *dec)
/* Converts the string to DEC. in degrees */
{
  if (rd_parse(buf, dec)==0) {
    if (*dec>=-90.0 && *dec<=90.0)
      return 0;
    else
      return MPK_INVALID_PAR;
 } else
   return MPK_INVALID_PAR;
}

void cmpack_dectostr(double dec, char *buf)
/* Converts the DEC. into the string */
{
  long x;
  if (dec>=0) {
	  x = (long)(dec*3600.0+0.5);
      sprintf(buf, "+%ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
  } else {
	  x = (long)(-dec*3600.0+0.5);
      sprintf(buf, "-%ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
  }
}

int cmpack_strtolon(const char *buf, double *dec)
/* Converts the string to longitude in degrees */
{
  if (rd_parse(buf, dec)==0) {
    if (*dec>=-180.0 && *dec<=180.0)
      return 0;
    else
      return MPK_INVALID_PAR;
 } else
   return MPK_INVALID_PAR;
}

void cmpack_lontostr(double lon, char *buf)
/* Converts the longitude into the string */
{
  long x;
  if (lon>=0) {
	  x = (long)(lon*3600.0+0.5);
      sprintf(buf, "E %ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
  } else {
	  x = (long)(-lon*3600.0+0.5);
      sprintf(buf, "W %ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
  }
}

int cmpack_strtolat(const char *buf, double *lat)
/* Converts the string to latitude in degrees */
{
  if (rd_parse(buf, lat)==0) {
    if (*lat>=-90.0 && *lat<=90.0)
      return 0;
    else
      return MPK_INVALID_PAR;
 } else
   return MPK_INVALID_PAR;
}

void cmpack_lattostr(double lat, char *buf)
/* Converts the latitude into the string */
{
  long x;
  if (lat>=0) {
	  x = (long)(lat*3600.0+0.5);
      sprintf(buf, "N %ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
  } else {
	  x = (long)(-lat*3600.0+0.5);
      sprintf(buf, "S %ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
  }
}

void cmpack_aztostr(double az, char *buf)
/* Converts the azimuth into the string */
{
  	long i, x;
	const char *dirs[16] = {"S", "SSW", "SW", "WSW", "W", "WNW", "NW", "NNW", \
							"N", "NNE", "NE", "ENE", "E", "ESE", "ES", "SSE"};
  	// To 0..360 degrees
  	az = az/360.0;
  	az = 360.0*(az-floor(az));
  	i  = (int)((az+11.25)/22.5);
  	if (i<0) i += 16; if (i>=16) i -= 16;
  	x = (long)(az*3600.0+0.5);
  	sprintf(buf, "%ld %02ld %02ld (%s)", (x/3600), ((x/60)%60), (x%60), dirs[i]);
}

void cmpack_alttostr(double alt, char *buf)
/* Converts the altitude into the string */
{
  long x;
  if (alt>=0) {
	  x = (long)(alt*3600.0+0.5);
      sprintf(buf, "%ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
  } else {
	  x = (long)(-alt*3600.0+0.5);
      sprintf(buf, "-%ld %02ld %02ld", (x/3600), ((x/60)%60), (x%60));
  }
}

int rd_parse(const char *buf, double *val)
// Parses string in various formats to R.A./DEC./LON./LAT.
{
	int state = 0, sign = 1;
	char *aux = NULL, *ptr, *tail, *buf2;
	double h = 0.0, m = 0.0, s = 0.0;
	
	if (val) 
		*val = 0.0;
	if (!buf) 
		return MPK_INVALID_PAR;
	
	buf2 = (char *)malloc(strlen(buf)+1);
	strcpy(buf2, buf);
	ptr = buf2;
	
	do {
		switch (state) {
		  case 0:
			if (*ptr=='+' || *ptr=='N' || *ptr=='E') {
				sign = 1;
				state = 1;
			} else
			if (*ptr=='-' || *ptr=='W' || *ptr=='S') {
				sign = -1;
				state = 1;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    h = *ptr - '0';
			    state = 2;
		    } else
		    if (*ptr!=' ') {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 1:
		    if (*ptr>='0' && *ptr<='9') {
			    h = *ptr - '0';
			    state = 2;
		    } else
		    if (*ptr!=' ') {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 2:
		    if (*ptr==0) {
			    if (val) *val = sign*h;
			    free(buf2);
			    return 0;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    h = h*10.0 + (*ptr - '0');
			    state = 3;
		    } else
		    if (*ptr=='.') {
			    aux = ptr-1;
			    state = 4;
		    } else
		    if (*ptr==',') {
			    *ptr = '.';
			    aux = ptr-1;
			    state = 4;
		    } else
		    if (*ptr==' ') {
			    state = 5;
		    } else
		    if (*ptr==':') {
			    state = 6;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 3:
		    if (*ptr==0) {
			    if (val) *val = sign*h;
			    free(buf2);
			    return 0;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    m = (*ptr - '0');
			    state = 18;
		    } else
		    if (*ptr=='.') {
			    aux = ptr-2;
			    state = 4;
		    } else
		    if (*ptr==',') {
			    *ptr = '.';
			    aux = ptr-2;
			    state = 4;
		    } else
		    if (*ptr==' ') {
			    state = 5;
		    } else
		    if (*ptr==':') {
			    state = 6;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 4:
		    if (*ptr>='0' && *ptr<='9') {
			    if (val) *val = sign*strtod(aux, &tail);
			    free(buf2);
			    return 0;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 5:
		    if (*ptr==':') {
			    state = 6;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    m = (*ptr - '0');
			    state = 8;
		    } else
		    if (*ptr!=' ') {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 6:
		    if (*ptr>='0' && *ptr<='9') {
			    m = (*ptr - '0');
			    state = 8;
		    } else
		    if (*ptr!=' ') {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 8:
		    if (*ptr==0) {
			    if (val) *val = sign*(h + m/60.0);
			    free(buf2);
			    return 0;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    m = m*10.0 + (*ptr - '0');
			    state = 9;
		    } else
		    if (*ptr=='.') {
			    aux = ptr-1;
			    state = 10;
		    } else
		    if (*ptr==',') {
			    *ptr = '.';
			    aux = ptr-1;
			    state = 10;
		    } else
		    if (*ptr==' ') {
			    state = 11;
		    } else
		    if (*ptr==':') {
			    state = 12;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 9:
		    if (*ptr==0) {
			    if (val) *val = sign*(h+m/60.0);
			    free(buf2);
			    return 0;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    s = (*ptr - '0');
			    state = 13;
		    } else
		    if (*ptr=='.') {
			    aux = ptr-2;
			    state = 10;
		    } else
		    if (*ptr==',') {
			    *ptr = '.';
			    aux = ptr-2;
			    state = 10;
		    } else
		    if (*ptr==' ') {
			    state = 11;
		    } else
		    if (*ptr==':') {
			    state = 12;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 10:
		    if (*ptr>='0' && *ptr<='9') {
			    if (val) *val = sign*(h + strtod(aux, &tail)/60.0);
			    free(buf2);
			    return 0;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 11:
		    if (*ptr==':') {
			    state = 12;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    s = (*ptr - '0');
			    state = 13;
		    } else
		    if (*ptr!=' ') {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 12:
		    if (*ptr>='0' && *ptr<='9') {
			    m = (*ptr - '0');
			    state = 13;
		    } else
		    if (*ptr!=' ') {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 13:
		    if (*ptr==0 || *ptr==' ') {
			    if (val) *val = sign*(h + m/60.0 + s/3600.0);
			    free(buf2);
			    return 0;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    s = s*10.0 + (*ptr - '0'); state = 14;
		    } else
		    if (*ptr=='.') {
			    aux = ptr-1;
			    state = 15;
		    } else
		    if (*ptr==',') {
			    *ptr = '.';
			    aux = ptr-1;
			    state = 15;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 14:
		    if (*ptr==0 || *ptr==' ') {
			    if (val) *val = sign*(h + m/60.0 + s/3600.0);
			    free(buf2);
			    return 0;
		    } else
		    if (*ptr=='.') {
			    aux = ptr-2;
			    state = 15;
		    } else
		    if (*ptr==',') {
			    *ptr = '.';
			    aux = ptr-2;
			    state = 15;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 15:
		    if (*ptr>='0' && *ptr<='9') {
			    if (val) *val = sign*(h + m/60.0 + strtod(aux, &tail)/3600.0);
			    free(buf2);
			    return 0;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;

		  case 18:
		    if (*ptr==0) {
			    if (val) *val = sign*(h*10.0 + m);
			    free(buf2);
			    return 0;
		    } else
		    if (*ptr>='0' && *ptr<='9') {
			    m = m*10.0 + (*ptr - '0');
			    state = 9;
		    } else
		    if (*ptr=='.') {
			    aux = ptr-3;
			    state = 4;
		    } else
		    if (*ptr==',') {
			    *ptr = '.';
			    aux = ptr-3;
			    state = 4;
		    } else
		    if (*ptr==' ') {
			    h = h*10.0 + m;
			    state = 5;
		    } else
		    if (*ptr==':') {
			    h = h*10.0 + m;
			    state = 6;
		    } else {
			    free(buf2);
		        return MPK_INVALID_PAR;
	        }
		    break;
	    }
		ptr++;
	} while (*(ptr-1)!=0);
	free(buf2);
	return MPK_INVALID_PAR;
}
