/**************************************************************

dark.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fitsio.h>

#include "comfun.h"
#include "fitsfun.h"
#include "dark.h"

double dark_minvalue;        // Lowest good datum
double dark_maxvalue;        // Highest good datum
int   dark_nx;              // Width of dark-frame
int   dark_ny;              // Height of dark-frame
double *dark_ddata;          // Data of dark-frame
double *dark_sdata;          // Data of souce-frame
int   dark_overflow;        // Output data were greater than maxvalue
int   dark_underflow;       // Output data were smaller than minvalue
int   dark_scaling;         // Use dark-frame scaling
double dark_exptime;		// Dark frame exposure time

int dark_init(const char *drk, int mode)
// Initializes the variables and loads the dark-frame
{
	FILE *f;
    fitsfile *dfits = NULL;
    long naxes[2];
 	char buf[MAXLINE];
 	int  dstat = 0;
    int anyf, naxis, bitpix, simple, extend;
    long pcount, gcount;

    dark_nx = 0;
    dark_ny = 0;
    dark_ddata = NULL;
    dark_sdata = NULL;
    dark_exptime = 0.0;
    dark_scaling = (mode & MPK_DARK_SCALING);
	dark_minvalue = 0.0;
	dark_maxvalue = 65535.0;

    /* Check parameters */
    if (!drk) {
        PrintOut(0, "Missing name of the dark frame.");
 	    return MPK_INVALID_PAR;
    }
    if (strlen(drk)==0) {
        PrintOut(0, "Missing name of the dark frame.");
 	    return MPK_INVALID_PAR;
    }
    f = fopen(drk,"r");
    if (f==NULL) {
 	    PrintOut(1, "Dark frame file not found.");
 	    return DRK_CANNOT_OPEN_DRK;
    }
    fclose(f);

    /* Open dark file */
    sprintf(buf,"Using dark frame: %s",drk);
    PrintOut(0, buf);
    
    ffdkopn(&dfits,drk,READONLY,&dstat);
    if (dstat!=0) return dstat;

    /* Read and check dimensions */
	ffghpr(dfits,2,&simple,&bitpix,&naxis,naxes,&pcount,&gcount,&extend,&dstat);
	if (dstat!=0) return dstat;
	dark_nx = naxes[0];
	dark_ny = naxes[1];
	if ((dark_nx<=0)||(dark_ny<=0)) {
		ffclos(dfits,&dstat);
		return MPK_INVALID_SIZE;
	}
	/* Read and check exposure duration */
	if (dark_scaling) {
    	fits_gexp(dfits, &dark_exptime);
		if (dark_exptime<=0.0) {
			PrintOut(1, "Invalid exposure duration in the dark frame.");
			ffclos(dfits,&dstat);
			return DRK_INVALID_EXPTIME;
		} else {
    		sprintf(buf,"Exposure duration: %.2f s", dark_exptime);
    		PrintOut(0, buf);
		}
		// Is the dark frame scalable?
		ffgkyl(dfits,"SCALABLE",&dark_scaling,NULL,&dstat);
		if (dstat!=0) {
			dark_scaling = 0;
			dstat = 0;
		}
		if (dark_scaling)
			PrintOut(0, "This is scalable dark, using advanced calibration.");
		else
			PrintOut(0, "Dark frame is not scalable, using standard calibration!");
	}

	/* Allocate the memory for dark, scientific and output frames */
	dark_ddata = malloc(dark_nx*dark_ny*sizeof(double));
	dark_sdata = malloc(dark_nx*dark_ny*sizeof(double));
	if ((!dark_ddata)||(!dark_sdata)) {
		ffclos(dfits,&dstat);
		return MPK_MEMORY;
	}

	/* Read dark frame data */
    ffg2dd(dfits,1,0.0,dark_nx,dark_nx,dark_ny,dark_ddata,&anyf,&dstat);
	if (dstat!=0) return dstat;

    /* Close dark file */
    ffclos(dfits,&dstat);
    if (dstat!=0) return dstat;
    
    /* Normal return */
    return 0;
}

int dark(const char *sci, const char *out)
// Computes the output image from source image and dark image (out=sci-dark)
{
 FILE *f;
 int anyf, i, naxis, bitpix, simple, extend, max;
 double exptime, scale;
 long naxes[2];
 long pcount, gcount;
 int  sstat = 0, ostat = 0;
 fitsfile *sfits, *ofits;
 char buf[MAXLINE];

 if (out) sprintf(buf,"(%s) => (%s)",sci,out);
     else sprintf(buf,"(%s)",sci);
 PrintOut(0, buf);

  /* Check source file */
  if (!sci) {
      PrintOut(0, "Missing name of the source image file.");
 	  return MPK_INVALID_PAR;
  }
  if (strlen(sci)==0) {
      PrintOut(0, "Missing name of the source image file.");
 	  return MPK_INVALID_PAR;
  }
  f = fopen(sci,"r");
  if (f==NULL) {
      PrintOut(1, "Source image file not found.");
      return MPK_CANNOT_OPEN_SRC;
  }
  fclose(f);

 /* Open exposure file */
 ffdkopn(&sfits,sci,(out?READONLY:READWRITE),&sstat);
 if (sstat!=0) return sstat;

 /* Read and check dimensions */
 ffghpr(sfits,2,&simple,&bitpix,&naxis,naxes,&pcount,&gcount,&extend,&sstat);
 if (sstat!=0) return sstat;
 if ((naxes[0]<=0)||(naxes[1]<=0)) {
	 ffclos(sfits,&sstat);
     return MPK_INVALID_SIZE;
 }
 if ((dark_nx!=naxes[0])||(dark_ny!=naxes[1])) {
	 ffclos(sfits,&sstat);
	 return DRK_DIFFERENET_SIZE;
 }
 
	/* Read and check exposure duration */
	if (dark_scaling) {
    	fits_gexp(sfits, &exptime);
		if (exptime<=0.0) {
			PrintOut(1, "Invalid exposure duration in the image file.");
			ffclos(sfits,&sstat);
			return MPK_INVALID_EXPTIME;
		} else {
    		sprintf(buf,"Exposure duration: %.2f s", exptime);
    		PrintOut(0, buf);
		}
	}

  /* Get equivalent data type and determine the max. and min. value */ 
  ffgiet(sfits,&bitpix,&sstat);
  switch (bitpix) 
  {
  case BYTE_IMG:  
    dark_minvalue = 0.0;
    dark_maxvalue = 255.0;
    break;
  case SBYTE_IMG:  
    dark_minvalue = -128.0;
    dark_maxvalue = 127.0;
    break;
  case SHORT_IMG:
    dark_minvalue = -32768.0;
    dark_maxvalue = 32767.0;
    break;
  case USHORT_IMG:
    dark_minvalue = 0.0;
    dark_maxvalue = 65535.0;
    break;
  case LONG_IMG:
    dark_minvalue = -2147483648.0;
    dark_maxvalue = 2147483647.0;
    break;
  case ULONG_IMG:
    dark_minvalue = 0.0;
    dark_maxvalue = 4294967296.0;
    break;
  default:
    dark_minvalue = 0.0;
    dark_maxvalue = 65535.0;
  }
  sprintf(buf, "Min. value   : %.1f", dark_minvalue); 
  PrintOut(1, buf);
  sprintf(buf, "Max. value   : %.1f", dark_maxvalue); 
  PrintOut(1, buf);

 /* Read exposure data */
 ffg2dd(sfits,1,0.0,dark_nx,dark_nx,dark_ny,dark_sdata,&anyf,&sstat);
 if (sstat!=0) {
	 ffclos(sfits,&sstat);
	 return sstat;
 }

 /* dark subtraction and range checking */
 max = naxes[0]*naxes[1];
 dark_underflow = dark_overflow = 0;
 scale = (dark_scaling ? exptime/dark_exptime : 1.0);
 for (i=0;i<max;i++) {
	  dark_sdata[i]=(double)(dark_sdata[i] - (scale*dark_ddata[i]));
	  if (dark_sdata[i]<dark_minvalue) {
		  dark_sdata[i] = dark_minvalue;
		  dark_underflow = 1;
	  }
	  if (dark_sdata[i]>dark_maxvalue) {
		  dark_sdata[i] = dark_maxvalue;
		  dark_overflow = 1;
	  }
 }

 if (out) {
 	/* output initialisation - parameters by scientific image */
 	remove(out);
 	ffdkinit(&ofits,out,&ostat);
 	if (ostat!=0) return MPK_CANNOT_OPEN_OUT;
 	ffcopy(sfits,ofits,1,&ostat);
 	ffphis(ofits,"Darkframe subtracted.",&ostat);
 	/* corrected image output */
 	ffp2dd(ofits,1,dark_nx,dark_nx,dark_ny,dark_sdata,&ostat);
 	if (ostat!=0) return ostat;
	/* Close output file */
 	ffclos(ofits,&ostat);
 	if (ostat!=0) return ostat;
 } else {
	 /* overwrite input image */
   	 ffphis(sfits,"Darkframe subtracted.",&sstat);
  	 ffp2dd(sfits,1,dark_nx,dark_nx,dark_ny,dark_sdata,&sstat);
  	 if (sstat!=0) return sstat;
 }

  /* Close exposure file */
  ffclos(sfits,&sstat);
  if (sstat!=0) return sstat;

  /* Normal return */
  if (dark_overflow) {
	  sprintf(buf,"An overflow has been occurred during computation (max.=%.1f).", dark_maxvalue);
	  PrintOut(1, buf);
  }
  if (dark_underflow) {
   	  sprintf(buf,"An underflow has been occurred during computation (min.=%.1f).", dark_minvalue);
	  PrintOut(1, buf);
  }
  return 0;
}

int dark_clean(void)
// Frees allocated memory
{
 if (dark_ddata) free(dark_ddata);
 if (dark_sdata) free(dark_sdata);
 return 0;
}
