/**************************************************************

flat.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fitsio.h>

#include "comfun.h"
#include "robmean.h"
#include "flat.h"

double flat_minvalue;    // Lowest good datum
double flat_maxvalue;    // Highest good datum
int   flat_nx;          // Width of flat-frame
int   flat_ny;          // Height of flat-frame
double *flat_fdata;      // flat frame
double *flat_sdata;      // scientific data
int   flat_overflow;    // Output data were greater than maxvalue
int   flat_underflow;   // Output data were smaller than minvalue
int   flat_divzero;     // Flat contains zero values

int flat_init(const char *flt)
// Initializes the variables and loads the dark-frame
{
	FILE *f;
    fitsfile *ffits = NULL;
 	char buf[MAXLINE];
    double fsig, fmed;
    int anyf, naxis, bitpix, simple, extend, max, i, fstat = 0;
    long pcount, gcount, naxes[2];

    flat_nx = 0;
    flat_ny = 0;
    flat_fdata = NULL;
    flat_sdata = NULL;
	flat_minvalue = 0.0;
	flat_maxvalue = 65535.0;

    /* Check flat file */
    if (!flt) {
        PrintOut(0, "Missing name of the flat frame.");
 	    return MPK_INVALID_PAR;
    }
    if (strlen(flt)==0) {
        PrintOut(0, "Missing name of the flat frame.");
 	    return MPK_INVALID_PAR;
    }
    f = fopen(flt,"r");
    if (f==NULL) {
 	    PrintOut(1, "Flat frame file not found.");
 	    return FLT_CANNOT_OPEN_FLT;
    }
    fclose(f);

    /* Open flat file */
	sprintf(buf,"Using flat frame: %s",flt);
    PrintOut(0, buf);
    ffdkopn(&ffits,flt,READONLY,&fstat);
    if (fstat!=0) return fstat;

    /* Read flat attributes */
	ffghpr(ffits,2,&simple,&bitpix,&naxis,naxes,&pcount,&gcount,&extend,&fstat);
	flat_nx = naxes[0];
	flat_ny = naxes[1];
	if ((flat_nx<=0)||(flat_ny<=0)) {
		ffclos(ffits,&fstat);
		return MPK_INVALID_SIZE;
	}

	/* Allocate the memory for flat, scientific frames */
	flat_fdata = malloc(flat_nx*flat_ny*sizeof(double));
	flat_sdata = malloc(flat_nx*flat_ny*sizeof(double));
	if ((!flat_fdata)||(!flat_sdata)) {
		ffclos(ffits,&fstat);
		return MPK_MEMORY;
	}

	/* Read flat frame data */
    ffg2dd(ffits,1,0.0,flat_nx,flat_nx,flat_ny,flat_fdata,&anyf,&fstat);
	if (fstat!=0) {
		ffclos(ffits,&fstat);
		return fstat;
    }

    /* Close flat file */
    ffclos(ffits,&fstat);

    /* median computation */
    max = flat_nx*flat_ny;
	cmpack_robustmeanD(max,flat_fdata,&fmed,&fsig);
    sprintf(buf,"Median       : %.4f",fmed);
    PrintOut(1, buf);
	sprintf(buf,"Skysig       : %.4f",fsig);
	PrintOut(1, buf);
	if (fmed==0) {
	    free(flat_fdata);
	    flat_fdata=NULL;
    	return FLT_MEAN_ZERO;
    }
    /* divide all flat values by median of the frame */
    for (i=0;i<max;i++)
        flat_fdata[i] = flat_fdata[i] / fmed;

    /* Normal return */
    return 0;
}

int flat(const char *sci, const char *out)
// Computes the output image from source image and flat image (out=mean*sci/flat)
{
  int anyf, i, naxis, bitpix, simple, extend, max;
  FILE *f;
  long naxes[2];
  long pcount, gcount;
  fitsfile *sfits;
  fitsfile *ofits;
  int  sstat = 0, ostat = 0;
  char buf[MAXLINE];

  if (out) sprintf(buf,"(%s) => (%s)",sci,out);
      else sprintf(buf,"(%s)",sci);
  PrintOut(0, buf);

  /* Check source file */
  if (!sci) {
      PrintOut(0, "Missing name of the source image file.");
 	  return MPK_INVALID_PAR;
  }
  if (strlen(sci)==0) {
      PrintOut(0, "Missing name of the source image file.");
 	  return MPK_INVALID_PAR;
  }
  if ((f=fopen(sci,"r"))==NULL) {
      PrintOut(1, "Source image file not found.");
      return MPK_CANNOT_OPEN_SRC;
  }
  fclose(f);

  /* Open exposure (source) file */
  if (out!=NULL) ffdkopn(&sfits,sci,READONLY,&sstat);
            else ffdkopn(&sfits,sci,READWRITE,&sstat);
  if (sstat) return sstat;

  /* Read source attributes */
  ffghpr(sfits,2,&simple,&bitpix,&naxis,naxes,&pcount,&gcount,&extend,&sstat);
  if ((naxes[0]<=0)||(naxes[1]<=0)) {
      ffclos(sfits,&sstat);
	  return MPK_INVALID_SIZE;
  }
  if ((naxes[0]!=flat_nx)||(naxes[1]!=flat_ny)) {
      ffclos(sfits,&sstat);
	  return FLT_DIFFERENET_SIZE;
  }
  
  /* Get equivalent data type and determine the max. and min. value */ 
  ffgiet(sfits,&bitpix,&sstat);
  switch (bitpix) 
  {
  case BYTE_IMG:  
    flat_minvalue = 0.0;
    flat_maxvalue = 255.0;
    break;
  case SBYTE_IMG:  
    flat_minvalue = -128.0;
    flat_maxvalue = 127.0;
    break;
  case SHORT_IMG:
    flat_minvalue = -32768.0;
    flat_maxvalue = 32767.0;
    break;
  case USHORT_IMG:
    flat_minvalue = 0.0;
    flat_maxvalue = 65535.0;
    break;
  case LONG_IMG:
    flat_minvalue = -2147483648.0;
    flat_maxvalue = 2147483647.0;
    break;
  case ULONG_IMG:
    flat_minvalue = 0.0;
    flat_maxvalue = 4294967296.0;
    break;
  default:
    flat_minvalue = 0.0;
    flat_maxvalue = 65535.0;
  }
	sprintf(buf, "Min. value   : %.1f", flat_minvalue); 
	PrintOut(1, buf);
	sprintf(buf, "Max. value   : %.1f", flat_maxvalue); 
	PrintOut(1, buf);

  /* Read exposure data */
  ffg2dd(sfits,1,0.0,naxes[0],naxes[0],naxes[1],flat_sdata,&anyf,&sstat);
  if (sstat) {
      ffclos(sfits,&sstat);
      return sstat;
  }

  /* flat subtraction and range checking */
  max = flat_nx*flat_ny;
  flat_underflow = flat_overflow = flat_divzero = 0;
  for (i=0;i<max;i++) {
      if (flat_fdata[i]>0.00001) {
	      flat_sdata[i]=flat_sdata[i]/flat_fdata[i];
      } else {
	      flat_sdata[i]=0.0;
	      flat_divzero = 1;
      }
	  if (flat_sdata[i]<flat_minvalue) {
		  flat_sdata[i] = flat_minvalue;
		  flat_underflow = 1;
	  }
	  if (flat_sdata[i]>flat_maxvalue) {
		  flat_sdata[i] = flat_maxvalue;
		  flat_overflow = 1;
	  }
  }

  /* Write output data */
  if (out) {
      /* output initialisation - parameters by scientific image */
 	  remove(out);
 	  ffdkinit(&ofits,out,&ostat);
 	  if (ostat!=0) {
          ffclos(sfits,&sstat);
	 	  return ostat;
 	  }
 	  ffcopy(sfits,ofits,1,&ostat);
 	  ffphis(ofits,"Flatframe corrected.",&ostat);
 	  /* corrected image output */
 	  ffp2dd(ofits,1,flat_nx,flat_nx,flat_ny,flat_sdata,&ostat);
 	  if (ostat!=0) {
          ffclos(sfits,&sstat);
          ffclos(ofits,&ostat);
	 	  return ostat;
 	  }
	  /* Close output file */
 	  ffclos(ofits,&ostat);
  } else {
   	  ffphis(sfits,"Flatframe corrected.",&sstat);
	  /* overwrite input image */
  	  ffp2dd(sfits,1,flat_nx,flat_nx,flat_ny,flat_sdata,&sstat);
  	  if (sstat!=0) {
          ffclos(sfits,&sstat);
	  	  return sstat;
  	  }
  }

  /* Close exposure file */
  ffclos(sfits,&sstat);

  /* Normal return */
  /*if (flat_divzero)
     return FLT_DIVZERO;
  if (flat_overflow) {
     sprintf(buf,"An overflow has been occurred during computation (max.=%.1f).", flat_maxvalue);
     PrintOut(0, buf);
     return MPK_DATA_OVERFLOW;
  }
  if (flat_underflow) {
   	 sprintf(buf,"An underflow has been occurred during computation (min.=%.1f).", flat_minvalue);
     PrintOut(0, buf);
     return MPK_DATA_UNDERFLOW;
  }*/
  return 0;
}

int flat_clean(void)
/* Free allocated memory */
{
 if (flat_fdata) free(flat_fdata);
 if (flat_sdata) free(flat_sdata);
 return 0;
}
