/**************************************************************

flatbat.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fitsio.h>

#ifdef _WIN32
#include <io.h>
#endif

#include "comfun.h"
#include "flat.h"

void PrintHelp()
{
 printf("FLATBAT version %s - utility for the flat frame correction.\n\n",VERSION);
 printf(PROJECTLINE);
 printf("\nUsage:flatbat [options] [@dirfile] [flat=flatframe] [mask=?] [image1] ...\n\n");
 printf("parameters:\n");
 printf("         @dirfile= file with image names for flat substract\n");
 printf("                   (for only '@', read image names from standart input)\n");
 printf("         flat=     flat frame image (default = %s)\n",FLATNAME);
 printf("         image1... names of simple images to flat correction\n");
 printf("         mask=     output mask (default = %s)\n",MASKNAME);
 printf("                   give 'mask=.' for overwrite input files (carefully!)\n");
 printf("options:\n");
 printf("        -f<file>   flat frame image (equivalent with 'flat=')\n");
 printf("        -o<mask>   output mask (equivalent with 'mask=')\n");
 printf("        -c<number> counter initial value (default = 1)\n");
 printf("        -q         quiet mode\n");
 printf("        -h         give this help\n");
 printf("        -v         print software version\n");
 printf("        -l         display software license\n");
 printf(WARRANTYLINE);
 printf(HOMEPAGELINE);
 printf(BUGREPORTS);
}

void report(int status, const char *file)
{
 if (cmpack_debug) 
 	fprintf(stderr, "The following error(s) has been occurred: %d\n",status);
 if (status<1000) 
 	ffrprt(stderr, status);
 else  
 	PrintError(PROGNAME, file, 0, status);
}

void error(int status, const char *file)
{
 report(status, file);
 exit(status);
}

int main(int argc, char **argv)
{
	FILE *f;
	int  res,i, icit, icit0 = 1, stdinput = 0, level = 0;
	char format[MAXLINE], outf[MAXLINE], line[MAXLINE], buf1[MAXLINE];
	char *namsci = "", *flt = FLATNAME, *mask = MASKNAME, *par;

	/* Zpracovani parametru prikazove radky */
	for (i=1;i<argc;i++) {
	   par = argv[i];
	   /* Prints help */
	   if ((strcmp(par,"-h")==0)||(strcmp(par,"--help")==0)) {
		  PrintHelp();
          return 0;
       }
	   /* Prints version */
	   if ((strcmp(par,"-v")==0)||(strcmp(par,"--version")==0)) {
		  PrintVersion(PROGNAME);
          return 0;
       }
       /* Prints license */
       if ((strcmp(par,"-l")==0)||(strcmp(par,"--license")==0)) {
          PrintLicense();
          return 0;
       }
       /* Flag for input from stdin or dirfile */
       if (strchr(par,'@')==par) {
	       namsci = malloc(strlen(par));
	       strcpy(namsci,&par[1]);
           stdinput = strlen(namsci)==0;
       }
       /* File with flat frame */
       if (strstr(par,"flat=")==par) {
	       flt = malloc(strlen(par)-4);
	       strcpy(flt,&par[5]);
       }
       if (strstr(par,"-f")==par) {
	       flt = malloc(strlen(par)-1);
	       strcpy(flt,&par[2]);
       }
       /* Mask of output files */
       if (strstr(par,"mask=")==par) {
	       mask = malloc(strlen(par)-4);
	       strcpy(mask,&par[5]);
       }
       if (strstr(par,"-o")==par) {
	       mask = malloc(strlen(par)-1);
	       strcpy(mask,&par[2]);
       }
       /* Initial counter value */
       if (strstr(par,"-c")==par) {
	       icit0 = atol(&par[2]);
       }
       /* Quiet/verbose flags */
       if (strcmp(par,"-q")==0 || strcmp(par,"--quiet")==0 || strcmp(par,"--silent")==0) {
           level = MPK_LEVEL_QUIET;
       }
       if (strcmp(par,"--verbose")==0 || strcmp(par,"--debug")==0) {
           level = MPK_LEVEL_DEBUG;
       }
  }

  cmpack_setoutput(PrintMessage, level);
  PrintPrologue(PROGNAME);
  res=flat_init(flt);
  if (res!=0) error(res, NULL);

  /* prepare format string for generating output file names */
  if (strcmp(mask,".")==0) {
     mask = NULL;            /* dot "." as output mask */
  } else {
     MaskToFormat(format,mask);
  }

  /* counter initialization */
  icit = icit0;

  /* image names reads from command line */
  for (i=1;i<argc;i++) {
	  par = argv[i];
      if ((strchr(par,'=')==0)&&(strchr(par,'@')!=par)&&(strchr(par,'-')!=par)) {
#ifdef _WIN32
		intptr_t fh;
    	struct _finddata_t fd;
		if ((fh=_findfirst(par, &fd))!=-1) {
			do {
				if ((fd.attrib & _A_SUBDIR)==0)  {
          			sprintf(outf,format,icit);
   	      			if (mask) {
		      			sprintf(outf,format,icit);
	      	  			res=flat(fd.name,outf);
          			} else {
	          			res=flat(fd.name,NULL);
          			}
          			if (res!=0) report(res, fd.name);
          			icit++;
          		}
          	} while (_findnext(fh, &fd)==0);
			_findclose(fh);
      	 } else {
	      	 report(MPK_CANNOT_OPEN_SRC, par);
      	 }
#else
		sprintf(outf,format,icit);
		if (mask) {
			sprintf(outf,format,icit);
			res=flat(par,outf);
		} else {
			res=flat(par,NULL);
		}
		if (res!=0) report(res, par);
		icit++;
#endif
      }
  }

  /* image names reads from dirfile */
  if (strlen(namsci)>0) {
	  f = fopen(namsci,"r");
	  if (f==NULL) error(MPK_NO_DIRFILE, namsci);
      while(fgets(buf1,MAXLINE,f)!=NULL) {
          sscanf(buf1,"%s",line);
          if (strcmp(line,"")) {
	      	  if (mask) {
		      	  sprintf(outf,format,icit);
	      	  	  res=flat(line,outf);
              } else {
	              res=flat(line,NULL);
              }
              if (res!=0) report(res, line);
              icit++;
          }
      }
  }

  /* Image names reads from stdin */
  if (stdinput) {
      while(fgets(buf1,MAXLINE,stdin)!=NULL) {
          sscanf(buf1,"%s",line);
          if (strcmp(line,"")) {
	      	  if (mask) {
		      	  sprintf(outf,format,icit);
	      	  	  res=flat(line,outf);
              } else {
	              res=flat(line,NULL);
              }
              if (res!=0) report(res, line);
              icit++;
          }
      }
  }

  if (icit==icit0) 
      PrintOut(0, "No input files.");

  res=flat_clean();
  if (res!=0) error(res, NULL);

#ifdef _WIN32
  //FitsFreeLibrary();
#endif

  return 0;
}
