/**************************************************************

konvbat.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fitsio.h>

#ifdef _WIN32
#include <io.h>
#endif

#include "comfun.h"
#include "iost.h"
#include "konve.h"

void PrintHelp()
{
 printf("KONVE version %s - utility for image file conversion\n",VERSION);
 printf(PROJECTLINE);
 printf("\nUsage:konve [options] [parameters] [@dirfile] [image1] ...\n\n");
 printf("parameters:\n");
 printf("     @dirfile=  file with names of images to convert\n");
 printf("                (for only '@', read images names from standart input)\n");
 printf("     par=file   parameter file (default is none)\n");
 printf("     tcor=sec   correction of time in seconds (can be real number)\n");
 printf("     flip=xy    flipping around axis ('x', 'y' or 'xy') \n");
 printf("     mask=name  mask of names of output files (default = %s)\n",OUTFILE);
 printf("     image1...  list of names of images to convert\n");
 printf("options:\n");
 printf("     -p<name>   parameter file (equivalent with 'par=')\n");
 printf("     -o<mask>   output mask (equivalent with 'mask=')\n");
 printf("     -c<number> counter initial value (default = 1)\n");
 printf("     -flip<xy>  flipping around axis (equivalent with 'flip=') \n");
 printf("     -q         quiet mode\n");
 printf("     -header    create only header of FITS file\n");
 printf("     -info      print short info only\n");
 printf("     -details   print detailed info only\n");
 printf("     -h         give this help\n");
 printf("     -l         display software license\n");
 printf("     -v         prints software version\n");
 printf("     -g         creates the example of the parameter file\n\n");
 printf(WARRANTYLINE);
 printf(HOMEPAGELINE);
 printf(BUGREPORTS);
}

void report(int status, const char *file)
{
 if (cmpack_debug) 
 	fprintf(stderr, "The following error(s) has been occurred: %d\n",status);
 if (status<1000) 
 	ffrprt(stderr, status);
 else  
 	PrintError(PROGNAME, file, 0, status);
}

void error(int status, const char *file)
{
 report(status, file);
 exit(status);
}

void CreatePar()
/* Creates a sample parameter file */
{
 FILE *fp;

 if ((fp=fopen(SMPFILE,"w"))!=NULL) {
   printf("Writing '%s' ...\n\n",SMPFILE);
   fprintf(fp,"OBJECT   = Star            # Object's designation\n");
   fprintf(fp,"OBSERVER = J. Walker       # Name of observer\n");
   fprintf(fp,"TELESCOP = 1.0m            # Telescope diameter\n");
   fprintf(fp,"RA       = 90.0            # R.A.\n");
   fprintf(fp,"DEC      = 90.0            # DEC.\n");
   fprintf(fp,"EQUINOX  = 2000.0          # Equinox\n");
   fclose(fp);
   printf("'%s' is an example of the parameter file. Rename it to 'konve.ini' and edit the parameters.\n\n",SMPFILE);
   printf("Note: If you want to use the parameter file, you have to specify its name explicitly on command line, see help or documentation for further details.\n");
 }
}

int main (int argc, char *argv[]) {
  FILE *f;
  int  i,icit,icit0,stdinput,mode,flipx,flipy,res, level = 0;
  double tcor = 0.0;
  char *par, line[MAXLINE], outf[MAXLINE], buf1[MAXLINE], format[MAXLINE];
  char *mask = OUTFILE;
  char *parname = NULL;
  char *dirfile = "";

  flipx = flipy = stdinput = mode = 0;
  icit = icit0 = 1;

  for( i = 0; i < argc; i++ ) {    /* loop over all parameters */
       par = argv[i];
	   /* Prints help */
	   if ((strcmp(par,"-h")==0)||(strcmp(par,"--help")==0)) {
		   PrintHelp();
           return 0;
       }
	   /* Prints version */
	   if ((strcmp(par,"-v")==0)||(strcmp(par,"--version")==0)) {
		  PrintVersion(PROGNAME);
          return 0;
       }
       /* Prints license */
       if ((strcmp(par,"-l")==0)||(strcmp(par,"--license")==0)) {
          PrintLicense();
          return 0;
       }
       /* Generate sample parameter file */
       if (strcmp(argv[i],"-g")==0) {
           CreatePar();
           return 0;
       }
       /* Dir file */
       if (strchr(par,'@')==par) {
	       dirfile = malloc(strlen(par));
	       strcpy(dirfile,&par[1]);
           stdinput = strlen(dirfile)==0;
       }
       /* Parameter file */
       if (strstr(par,"par=")==par) {
	       parname = malloc(strlen(par)-3);
	       strcpy(parname,&par[4]);
       }
       if (strstr(argv[i],"-p")==par) {
	       parname = malloc(strlen(par)-1);
	       strcpy(parname,&par[2]);
       }
       /* time correction */
       if (strstr(par,"-t")==par) {
	       tcor = atof(&par[2]);
       }
       if (strstr(par,"tcor=")==par) {
	       tcor = atof(&par[5]);
       }
       /* Output files */
       if (strstr(par,"mask=")==par) {
	       mask = malloc(strlen(par)-4);
	       strcpy(mask,&par[5]);
       }
       if (strstr(argv[i],"-o")==par) {
	       mask = malloc(strlen(par)-1);
	       strcpy(mask,&par[2]);
       }
       /* Flipping flags */
       if ((strstr(par,"-flip")==par)||(strstr(argv[i],"flip=")==par)) {
	       flipx = strchr(par,'x')||strchr(par,'X');
	       flipy = strchr(par,'y')||strchr(par,'Y');
       }
       /* Initial counter value */
       if (strstr(par,"-c")==par) {
	       icit0 = atol(&par[2]);
       }
       /* Write only header */
       if (strcmp(argv[i],"-header")==0) {
           mode = 1;
       }
       /* Write only short info */
       if (strcmp(argv[i],"-info")==0) {
           mode = 2;
       }
       /* Write only detailed info */
       if (strcmp(argv[i],"-details")==0) {
           mode = 3;
       }
       /* Quiet/verbose flags */
       if (strcmp(par,"-q")==0 || strcmp(par,"--quiet")==0 || strcmp(par,"--silent")==0) {
           level = MPK_LEVEL_QUIET;
       }
       if (strcmp(par,"--verbose")==0 || strcmp(par,"--debug")==0) {
           level = MPK_LEVEL_DEBUG;
       }
  }

  cmpack_setoutput(PrintMessage, level);
  PrintPrologue(PROGNAME);
  res=konv_init(parname,flipx,flipy,tcor,mode);
  if (res!=0) error(res, NULL);

  /* prepare format string for generating output file names */
  MaskToFormat(format,mask);

  /* Init output filenames counter */
  icit = icit0;

  /* images names are given through the command line */
  for (i=1; i<argc; i++) {
  	par = argv[i];
    if ((strchr(par,'=')==0)&&(strchr(par,'@')!=par)&&(strchr(par,'-')!=par)) {
#ifdef _WIN32
  		struct _finddata_t fd;
		intptr_t fh;
		if ((fh=_findfirst(par, &fd))!=-1) {
			do {
				if ((fd.attrib & _A_SUBDIR)==0)  {
          			sprintf(outf,format,icit);
          			res = konv(fd.name,outf);
          			if (res!=0) report(res, fd.name);
          			icit++;
          		}
          	} while (_findnext(fh, &fd)==0);
			_findclose(fh);
      	 } else {
	      	 report(MPK_CANNOT_OPEN_SRC, par);
      	 }
#else
			sprintf(outf, format, icit);
			res = konv(par, outf);
			if (res!=0) report(res, par);
			icit++;
#endif
  	}
  }

  /* image names are read from dirfile */
  if (strlen(dirfile)>0) {
      f = fopen(dirfile,"r");
	  if (f==NULL) error(MPK_NO_DIRFILE, dirfile);
      while(fgets(buf1,MAXLINE,f)!=NULL) {
          sscanf(buf1,"%s",line);
          if (strcmp(line,"")) {
              sprintf(outf,format,icit);
              res=konv(line,outf);
   			  if (res!=0) report(res, line);
   			  icit++;
          }
      }
      fclose(f);
  }

  /* Image names are read from stdin */
  if (stdinput) {
      while(fgets(buf1,MAXLINE,stdin)!=NULL) {
          sscanf(buf1,"%s",line);
          if (strcmp(line,"")) {
              sprintf(outf,format,icit);
              res=konv(line,outf);
   			  if (res!=0) report(res, line);
   			  icit++;
          }
      }
  }

  if (icit==icit0) 
  	PrintOut(0, "No input files.");

  res = konv_clean();
  if (res!=0) error(res, NULL);

  return 0;
}
