/**************************************************************

muniphot.c (C-Munipack project)
Copyright (C) 2003 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <math.h>
#include <fitsio.h>

#ifdef _WIN32
#include <io.h>
#endif

#include "comfun.h"
#include "phot.h"

void PrintHelp()
{
 printf("MUNIPHOT version %s - Photometry of image series.\n",VERSION);
 printf(PROJECTLINE);
 printf("\nUsage: muniphot [options] [@dirfile] [image1] ..\n\n");
 printf("parameters:\n");
 printf("     @dirfile  file with input image names\n");
 printf("               (for only '@', read filenames from standart input)\n");
 printf("     image1... names of simple scientific exposures\n");
 printf("     par=file  file with parameters (default = %s)\n",OPTFILE);
 printf("     ap=file   file with aperture radii (default = %s)\n",APFILE);
 printf("options:\n");
 printf("     -p<file>  file with parameters (equivalent with 'par=')\n");
 printf("     -a<file>  file with aperture radii (equivalent with 'ap=')\n");
 printf("     -q        quiet mode\n");
 printf("     -h        give this help\n");
 printf("     -v        prints software version\n");
 printf("     -l        display software license\n");
 printf("     -g        creates the example of the parameter and aperture files\n");
 printf(WARRANTYLINE);
 printf(HOMEPAGELINE);
 printf(BUGREPORTS);
}

void report(int status, const char *file)
{
 if (cmpack_debug) 
 	fprintf(stderr, "The following error(s) has been occurred: %d\n",status);
 if (status<1000) 
 	ffrprt(stderr, status);
 else  
 	PrintError(PROGNAME, file, 0, status);
}

void error(int status, const char *file)
{
 report(status, file);
 exit(status);
}

void CreatePar()
/* Creates a sample parameter file and aperture file */
{
 FILE *fp;

 if ((fp=fopen(SOPFILE,"w"))!=NULL) {
   printf("Writing '%s' ...\n\n",SOPFILE);
   fprintf(fp,"READNS   = 15.00                 # Read noise (ADU; 1 frame)\n");
   fprintf(fp,"GAIN     = 2.30                  # Gain (e-/ADU; 1 frame)\n");
   fprintf(fp,"LODATA   = 7.0                   # Low good datum (sigmas)\n");
   fprintf(fp,"HIDATA   = 65535.0               # High good datum (ADU)\n");
   fprintf(fp,"FWHM     = 3.00                  # FWHM of object\n");
   fprintf(fp,"THRESH   = 2.50                  # Threshold (sigmas)\n");
   fprintf(fp,"LOSHARP  = 0.20                  # Low sharpness cutoff\n");
   fprintf(fp,"HISHARP  = 1.00                  # High sharpness cutoff\n");
   fprintf(fp,"LOROUND  = -1.00                 # Low roundness cutoff\n");
   fprintf(fp,"HIROUND  = 1.00                  # High roundness cutoff\n");
   fprintf(fp,"FR_AVG   = 1                     # Number of frames averaged\n");
   fprintf(fp,"FR_SUM   = 1                     # Number of frames summed\n");
   fprintf(fp,"SORT     = 4                     # Column used for sorting stars\n");
   fclose(fp);
   printf("'%s' is an example of the parameter file. Rename it to '%s' and edit the parameters.\n\n",SOPFILE,OPTFILE);
 }

 if ((fp=fopen(SAPFILE,"w"))!=NULL) {
   printf("Writing '%s' ...\n\n",SAPFILE);
   fprintf(fp,"A1       = 2.0                   # Aperture #1 (in pixels)\n");
   fprintf(fp,"A2       = 2.7273\n");
   fprintf(fp,"A3       = 3.8182\n");
   fprintf(fp,"A4       = 5.2727\n");
   fprintf(fp,"A5       = 7.0909\n");
   fprintf(fp,"A6       = 9.2727\n");
   fprintf(fp,"A7       = 11.8182\n");
   fprintf(fp,"A8       = 14.7273\n");
   fprintf(fp,"A9       = 18.0\n");
   fprintf(fp,"A10      = 21.6364\n");
   fprintf(fp,"A11      = 25.6364\n");
   fprintf(fp,"A12      = 30.0\n");
   fprintf(fp,"IS       = 20.0                  # Inner sky radius\n");
   fprintf(fp,"OS       = 30.0                  # Outer sky radius\n");
   fclose(fp);
   printf("'%s' is an example of the aperture file. Rename it to '%s' and edit the parameters.\n\n",SAPFILE,APFILE);
 }
}

int main(int argc, char **argv)
{
    int stdinput = FALSE, i, n, j, res, level = 0;
    double jd;
    char line[MAXLINE], buf1[MAXLINE], *par, outf[MAXLINE];
	char *namsci = "", *nampar = NULL, *namap  = NULL;
	FILE *f;

	/* Zpracovani parametru prikazove radky */
	for (i=1;i<argc;i++) {
	   par = argv[i];
	   /* Prints help */
	   if ((strcmp(par,"-h")==0)||(strcmp(par,"--help")==0)) {
		   PrintHelp();
          return 0;
       }
	   /* Prints version */
	   if ((strcmp(par,"-v")==0)||(strcmp(par,"--version")==0)) {
		  PrintVersion(PROGNAME);
          return 0;
       }
       /* Prints license */
       if ((strcmp(par,"-l")==0)||(strcmp(par,"--license")==0)) {
          PrintLicense();
          return 0;
       }
       /* Generate sample parameter file */
       if (strcmp(argv[i],"-g")==0) {
           CreatePar();
           return 0;
       }
       /* Flag for input from stdin or dirfile */
       if (strchr(par,'@')==par) {
	       namsci = malloc(strlen(par));
	       strcpy(namsci,&par[1]);
           stdinput = strlen(namsci)==0;
       }
       /* Parameter file */
       if (strstr(par,"par=")==par) {
	       nampar = malloc(strlen(par)-3);
	       strcpy(nampar,&par[4]);
       }
       if (strstr(argv[i],"-p")==par) {
	       nampar = malloc(strlen(par)-1);
	       strcpy(nampar,&par[2]);
       }
       /* Aperture file */
       if (strstr(par,"ap=")==par) {
	       namap = malloc(strlen(par)-2);
	       strcpy(namap,&par[3]);
       }
       if (strstr(argv[i],"-a")==par) {
	       namap = malloc(strlen(par)-1);
	       strcpy(namap,&par[2]);
       }
       /* Quiet/verbose flags */
       if (strcmp(par,"-q")==0 || strcmp(par,"--quiet")==0 || strcmp(par,"--silent")==0) {
           level = MPK_LEVEL_QUIET;
       }
       if (strcmp(par,"--verbose")==0 || strcmp(par,"--debug")==0) {
           level = MPK_LEVEL_DEBUG;
       }
  }

  n = 0;
  cmpack_setoutput(PrintMessage, level);
  PrintPrologue(PROGNAME);
  res=mphot_init(nampar,namap);
  if (res!=0) error(res, NULL);

  /* image names reads from command line */
  for (i=1;i<argc;i++) {
	  par = argv[i];
      if ((strchr(par,'=')==0)&&(strchr(par,'@')!=par)&&(strchr(par,'-')!=par)) {
#ifdef _WIN32
		  intptr_t fh;
		struct _finddata_t fd;
		 if ((fh=_findfirst(par, &fd))!=-1) {
			do {
				if ((fd.attrib & _A_SUBDIR)==0)  {
          			changeext(outf,fd.name,"srt");
          			res=mphot(fd.name,outf,&jd,&j);
          			if (res!=0) report(res, fd.name);
          			n++;
          		}
          	} while (_findnext(fh, &fd)==0);
			_findclose(fh);
      	 }
#else
		changeext(outf,par,"srt");
		res=mphot(par,outf,&jd,&j);
		if (res!=0) report(res, par);
		n++;
#endif
      }
  }

  /* image names reads from dirfile */
  if (strlen(namsci)>0) {
	  f = fopen(namsci,"r");
	  if (f==NULL) error(MPK_NO_DIRFILE, namsci);
      while(fgets(buf1,MAXLINE,f)!=NULL) {
          sscanf(buf1,"%s",line);
          if (strcmp(line,"")) {
    	      changeext(outf,line,"srt");
	          res=mphot(line,outf,&jd,&j);
   			  if (res!=0) report(res, line);
	          n++;
          }
      }
  }

  /* Image names reads from stdin */
  if (stdinput) {
      while(fgets(buf1,MAXLINE,stdin)!=NULL) {
          sscanf(buf1,"%s",line);
          if (strcmp(line,"")) {
    	      changeext(outf,line,"srt");
	          res=mphot(line,outf,&jd,&j);
   			  if (res!=0) report(res, line);
	          n++;
          }
      }
  }

  if (n<=0) 
      PrintOut(0, "No input files.");

  res=mphot_clean();
  if (res!=0) error(res, NULL);

  return(0);
}
